"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.run = exports.processDeploymentGroup = exports.processDeploymentTarget = void 0;
const aws_clients_1 = require("@takomo/aws-clients");
const core_1 = require("@takomo/core");
const util_1 = require("@takomo/util");
const child_process_1 = require("child_process");
const cockatiel_1 = require("cockatiel");
const path_1 = require("path");
const ramda_1 = __importDefault(require("ramda"));
const util_2 = require("util");
const execP = (0, util_2.promisify)(child_process_1.exec);
const getDeploymentRole = (currentIdentity, group, target, disableMapRole, roleName) => {
    if (disableMapRole) {
        return undefined;
    }
    const deploymentRole = target.deploymentRole ?? group.deploymentRole;
    if (deploymentRole) {
        return deploymentRole;
    }
    const deploymentRoleName = roleName ?? target.deploymentRoleName ?? group.deploymentRoleName;
    if (!deploymentRoleName) {
        return undefined;
    }
    const deploymentAccountId = target.accountId ?? currentIdentity.accountId;
    return {
        iamRoleArn: `arn:aws:iam::${deploymentAccountId}:role/${deploymentRoleName}`,
    };
};
const captureValue = ({ captureAfterLine, captureLastLine, captureBeforeLine, }, output) => {
    const lines = output.trim().split("\n");
    if (captureLastLine) {
        return (ramda_1.default.last(lines) ?? "") + "\n";
    }
    if (!captureAfterLine && !captureBeforeLine) {
        return output;
    }
    const start = captureAfterLine
        ? lines.findIndex((item) => item === captureAfterLine)
        : -1;
    const end = captureBeforeLine
        ? lines.findIndex((item) => item === captureBeforeLine)
        : -1;
    const endLineNumber = end === -1 ? lines.length : end;
    return lines
        .slice(start + 1, endLineNumber)
        .map((l) => `${l}\n`)
        .join("");
};
const runJsMapFunction = async ({ ctx, mapCommand, group, target, logger, credentials, mapArgs, }) => {
    const mapFunctionFullPath = (0, util_1.expandFilePath)(ctx.projectDir, mapCommand);
    logger.debug(`Run map function from file: ${mapFunctionFullPath}`);
    // eslint-disable-next-line
    const mapperFn = require(mapFunctionFullPath);
    return mapperFn({
        credentials,
        target: target,
        deploymentGroupPath: group.path,
        args: mapArgs,
    });
};
const runMapProcessCommand = async ({ ctx, mapCommand, group, target, logger, credentials, input, mapArgs, }) => {
    logger.debug(`Run map command: ${mapCommand}`);
    const targetJson = JSON.stringify(target);
    const additionalVariables = {
        TKM_TARGET_NAME: target.name,
        TKM_TARGET_JSON: targetJson,
        TKM_DEPLOYMENT_GROUP_PATH: group.path,
        TKM_MAP_ARGS: "",
    };
    if (mapArgs) {
        additionalVariables.TKM_MAP_ARGS =
            typeof mapArgs === "string" ? mapArgs : JSON.stringify(mapArgs);
    }
    const env = (0, aws_clients_1.prepareAwsEnvVariables)({
        env: process.env,
        credentials,
        additionalVariables,
    });
    const { stdout } = await execP(mapCommand, {
        env,
        cwd: ctx.projectDir,
    });
    return captureValue(input, stdout);
};
const runMapCommand = (props) => props.mapCommand.startsWith("js:")
    ? runJsMapFunction({ ...props, mapCommand: props.mapCommand.substr(3) })
    : runMapProcessCommand(props);
const runReduceProcessCommand = async ({ credentials, ctx, reduceCommand, targetResults, }) => {
    const additionalVariables = {
        TKM_TARGET_RESULTS_JSON: JSON.stringify(targetResults),
    };
    const env = (0, aws_clients_1.prepareAwsEnvVariables)({
        env: process.env,
        credentials,
        additionalVariables,
    });
    const { stdout } = await execP(reduceCommand, {
        env,
        cwd: ctx.projectDir,
    });
    return stdout;
};
const runJsReduceFunction = ({ credentials, ctx, reduceCommand, targetResults, }) => {
    const fullReduceFunctionPath = (0, util_1.expandFilePath)(ctx.projectDir, reduceCommand);
    // eslint-disable-next-line
    const reduceFn = require(fullReduceFunctionPath);
    return reduceFn({
        credentials,
        targets: targetResults,
    });
};
const runReduceCommand = async (props) => props.reduceCommand.startsWith("js:")
    ? runJsReduceFunction({
        ...props,
        reduceCommand: props.reduceCommand.substr(3),
    })
    : runReduceProcessCommand(props);
const getCredentialManager = async (ctx, deploymentRole) => deploymentRole
    ? await ctx.credentialManager.createCredentialManagerForRole(deploymentRole.iamRoleArn)
    : ctx.credentialManager;
const getCredentialsForMapCommand = async ({ ctx, input: { mapRoleName, disableMapRole } }, group, target) => ctx.credentialManager
    .getCallerIdentity()
    .then((currentIdentity) => getDeploymentRole(currentIdentity, group, target, disableMapRole, mapRoleName))
    .then((deploymentRole) => getCredentialManager(ctx, deploymentRole))
    .then((c) => c.getCredentials())
    .then(async (c) => {
    await c.getPromise();
    return c;
});
const getCredentialsForReduceCommand = async (ctx, reduceRoleArn) => {
    const credentialManager = reduceRoleArn
        ? await ctx.credentialManager.createCredentialManagerForRole(reduceRoleArn)
        : ctx.credentialManager;
    const credentials = await credentialManager.getCredentials();
    await credentials.getPromise();
    return credentials;
};
const processDeploymentTarget = async (props, group, target, timer, state, logger, mapArgs) => {
    const { ctx, input } = props;
    const { mapCommand } = input;
    if (state.failed) {
        timer.stop();
        return {
            message: "Cancelled",
            status: "CANCELLED",
            success: false,
            name: target.name,
            timer,
        };
    }
    logger.info("Start run");
    try {
        const credentials = await getCredentialsForMapCommand(props, group, target);
        const value = await runMapCommand({
            mapCommand,
            ctx,
            logger,
            target,
            group,
            credentials,
            input,
            mapArgs,
        });
        timer.stop();
        return {
            status: "SUCCESS",
            name: target.name,
            success: true,
            message: "Success",
            value,
            timer,
        };
    }
    catch (error) {
        timer.stop();
        logger.error(`Map command failed for target '${target.name}'`, error);
        return {
            status: "FAILED",
            name: target.name,
            success: false,
            message: "Error",
            error,
            timer,
        };
    }
};
exports.processDeploymentTarget = processDeploymentTarget;
const convertToOperation = (props, group, timer, state, target, results, policy, listener, mapArgs) => () => policy.execute(async () => {
    await listener.onTargetBegin();
    const result = await (0, exports.processDeploymentTarget)(props, group, target, timer.startChild(target.name), state, props.io.childLogger(target.name), mapArgs);
    results.push(result);
    await listener.onTargetComplete();
    return result;
});
const processDeploymentGroup = async (props, group, timer, state, listener, mapArgs) => {
    const { io, input } = props;
    const { concurrentTargets } = input;
    io.info(`Run deployment group: ${group.path}`);
    const results = new Array();
    const deploymentPolicy = cockatiel_1.Policy.bulkhead(concurrentTargets, 10000);
    const operations = group.targets.map((target) => convertToOperation(props, group, timer, state, target, results, deploymentPolicy, listener, mapArgs));
    await Promise.all(operations.map((o) => o()));
    timer.stop();
    return {
        ...(0, core_1.resolveCommandOutputBase)(results),
        results,
        path: group.path,
        timer,
    };
};
exports.processDeploymentGroup = processDeploymentGroup;
const convertArgs = async (projectDir, args) => {
    if (!args) {
        return undefined;
    }
    if (args.startsWith("file:")) {
        const filePath = args.substr(5);
        const fullFilePath = (0, util_1.expandFilePath)(projectDir, filePath);
        const contents = await (0, util_1.readFileContents)(fullFilePath);
        switch ((0, path_1.extname)(filePath)) {
            case ".json":
                return JSON.parse(contents);
            case ".yml":
            case ".yaml":
                return (0, util_1.parseYaml)(fullFilePath, contents);
            default:
                return contents;
        }
    }
    return args;
};
const run = async (props) => {
    const { input, plan, listener, ctx, io } = props;
    const childTimer = input.timer.startChild("run");
    const results = new Array();
    const mapArgs = await convertArgs(ctx.projectDir, input.mapArgs);
    const state = { failed: false };
    for (const group of plan.groups) {
        const result = await (0, exports.processDeploymentGroup)(props, group, childTimer.startChild(group.name), state, listener, mapArgs);
        results.push(result);
    }
    const outputBase = (0, core_1.resolveCommandOutputBase)(results);
    const targetResults = results
        .map((r) => r.results)
        .flat()
        .map((r) => r.value);
    if (!outputBase.success || !input.reduceCommand) {
        childTimer.stop();
        return {
            ...outputBase,
            result: targetResults,
            timer: childTimer,
            outputFormat: input.outputFormat,
        };
    }
    const reduceCredentials = await getCredentialsForReduceCommand(ctx, input.reduceRoleArn);
    try {
        const result = await runReduceCommand({
            targetResults,
            ctx,
            credentials: reduceCredentials,
            reduceCommand: input.reduceCommand,
        });
        childTimer.stop();
        return {
            ...outputBase,
            result,
            timer: childTimer,
            outputFormat: input.outputFormat,
        };
    }
    catch (error) {
        childTimer.stop();
        io.error(`Reduce command failed`, error);
        return {
            error,
            result: undefined,
            message: "Reduce command failed",
            success: false,
            status: "FAILED",
            timer: childTimer,
            outputFormat: input.outputFormat,
        };
    }
};
exports.run = run;
//# sourceMappingURL=run.js.map